library(R2jags); library(ggplot2)

# Please note that, for illustration purposes, the data provided in this R file are simulated
# and are not the same that are reported on in the manuscript. 
load('/Users/fonz/Desktop/DTMRsim.Rdata')

Y <- DTMRsim$ResponseMatrix # response data
Mcred <- DTMRsim$Mcred # credential status variable
Q <- DTMRsim$Qmatrix # DTMR Q matrix
LCDM <- DTMRsim$LCDM # LDCM function that is passed to JAGS

nPerson <- nrow(Y) # NUMBER OF EXAMINEES
nItems <- nrow(Q) # NUMBER OF ITEMS
nAttributes <- ncol(Q) # NUMBER OF ATTRIBUTES 
G <- Mcred
data.list <- list("nPerson", "nItems", "nAttributes", "Q", "Y", "G") # DATA TO BE PASSED TO JAGS

# JAGS WILL SAVE MARKOV CHAIN ITERATIONS FOR THESE PARAMETERS
track.parameters <- c(
  # ITEM PARAMETERS #
  "lambda0", "lambda0G",
  "lambda1", "lambda1G",
  "lambda2", "lambda2G",
  "lambda3", "lambda3G",
  "lambda4", "lambda4G",
  "lambda12", "lambda12G",
  "lambda13", "lambda13G",
  "lambda14", "lambda14G",
  "lambda23", "lambda23G",
  "lambda24", "lambda24G",
  "lambda34", "lambda34G",
  # ATTRIBUTE PARAMETERS #
  "beta0", "beta0G",
  "gamma0", "gamma0G",
  "gamma1AP", "gamma1APG",
  "delta0", "delta0G",
  "delta1AP", "delta1APG",
  "kappa0", "kappa0G",
  "kappa1PI", "kappa1PIG",
  "kappa2MC",  "kappa2MCG",
  "kappa12MCPI", "kappa12MCPIG",
  # ATTRIBUTE PROFILES #
  "alpha"
)

# ADD MCRED VARIABLES TO track.parameters (ONLY FOR OBSERVATIONS WITH MISSING Mcred VARIABLE)
track.parameters <- c(track.parameters, 
                      paste0("G[", which(is.na(G)), "]"))

# RUN THE JAGS MODEL
res <- jags.parallel(
  data = data.list,
  inits = NULL,
  parameters.to.save = track.parameters, 
  model.file = LCDM,
  n.chains = 2, 
  n.iter = 50,
  n.burnin = 25,
  n.thin = 1,
  DIC = TRUE
)

# DIAGNOSTICS ----------------------------------------------------
# NUMBER OF PARAMETERS THAT DO NOT DISPLAY EVIDENCE OF CONVERGENCE
sum(res$BUGSoutput$summary[-1, "Rhat"] > 1.10) 
max(res$BUGSoutput$summary[-1, "Rhat"]) # MAX Rhat
which(res$BUGSoutput$summary[-1, "Rhat"] > 1.10)
sum(is.na(res$BUGSoutput$summary[-1, "Rhat"]))

res.mcmc <- as.mcmc(res)
res.mat <- as.matrix(res.mcmc)
options(max.print=999999)

#------------------------------------------------------------------------------------------------------
# ANALYSIS OF CREDENTIAL STATUS -----------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------
cred.vec.names <- colnames(res.mat) %in% paste0("G[", which(is.na(G)), "]")
cred.mat <- res.mat[, cred.vec.names]
dim(cred.mat) # verify we extracted the correct variables

post.MCred.means <- round(colMeans(cred.mat), 3)
apply(cred.mat, 2, sd) # posterior SD
Teacher.ID <- paste0("ID: ", which(is.na(G)))
status <- ifelse(post.MCred.means<= 0.40, "No Credential", ifelse(post.MCred.means >= 0.60, "Credential", "Cannot be Determined"))
cred.df <- data.frame(post.mean = post.MCred.means, ID = Teacher.ID, status = status)
ggplot(cred.df, aes(x = ID, y = post.mean, fill = status)) + 
  geom_col() + theme_bw() + 
  scale_fill_manual(name = 'Credential Status', values = c('grey', 'blue', 'red')) + 
  geom_hline(yintercept = c(0.4, 0.6), linetype = 'dashed') + 
  xlab('Teacher ID') + ylab('Posterior Mean')

#------------------------------------------------------------------------------------------------------
# ANALYSIS OF ITEM PARAMETERS -------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------
res.mat.chain1 <- as.matrix(res.mcmc[[1]])
res.mat.chain2 <- as.matrix(res.mcmc[[2]])
res.mat <- as.matrix(res.mcmc)

par.names <- c(
  "lambda0", "lambda0G", 
  "lambda1", "lambda1G",
  "lambda2", "lambda2G",
  "lambda3", "lambda3G",
  "lambda4", "lambda4G",
  "lambda12", "lambda12G",
  "lambda13", "lambda13G",
  "lambda14", "lambda14G",
  "lambda23", "lambda23G",
  "lambda24", "lambda24G",
  "lambda34", "lambda34G"
)

item.par.names <- paste0(par.names, "[", rep(1:nItems, each = length(par.names)), "]")

# DATA FOR EACH CHAIN
item.post.chain1 <- res.mat.chain1[,item.par.names]
item.post.chain2 <-  res.mat.chain2[,item.par.names]

# REMOVE ITEM PARAMETERS THAT WERE NOT ESTIMATED
remove.pars <- which(colSums(item.post.chain1) == 0)
item.post.chain1 <- item.post.chain1[, -remove.pars]
item.post.chain2 <- item.post.chain2[, -remove.pars]

# DATA WITH BOTH CHAINS COMBINED
item.post.mat <- res.mat[,item.par.names]
item.post.mat <- item.post.mat[,-remove.pars]
keep.names <- colnames(item.post.mat)
keep.names
dim(item.post.mat)

# CREATE ODDS RATIO MATRIX
OR.post <- apply(item.post.mat, 2, exp)

# ANALYSIS OF PSRF
res$BUGSoutput$summary[keep.names, "Rhat"]

# POSTERIOR SUMMARY
post.means <- apply(item.post.mat, 2, mean)
post.OR.means <- apply(OR.post, 2, mean)
post.sd <- apply(item.post.mat, 2, sd)
post.OR.sd <- apply(OR.post, 2, sd)
post.lcred <- apply(item.post.mat, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[1])
post.ucred <- apply(item.post.mat, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[2])
post.lcred.OR <- apply(OR.post, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[1])
post.ucred.OR <- apply(OR.post, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[2])
Include0 <- NULL
for(i in 1:ncol(item.post.mat)){
  Include0[i] <- ifelse(post.lcred[i] < 0 & post.ucred[i] > 0, "Yes", "No")
}

post.summary <- cbind(post.means, post.sd, post.lcred, post.ucred) 
post.summary <- apply(post.summary, 2, function(x) as.numeric(sprintf('%.2f', x)))
post.summary.OR <- cbind(post.OR.means, post.OR.sd, post.lcred.OR, post.ucred.OR)
post.summary.OR <- apply(post.summary.OR, 2, function(x) as.numeric(sprintf('%.2f', x)))
post.summary <- cbind.data.frame(post.summary, 
                                 Include0, 
                                 post.summary.OR)
row.names(post.summary) <- keep.names
post.summary # CAN USE THIS TO MAKE PLOTS 

#------------------------------------------------------------------------------------------------------
# ANALYSIS OF STRUCTURAL MODEL PARAMETERS -------------------------------------------------------------
#------------------------------------------------------------------------------------------------------
str.pars <- c(  "beta0", "beta0G",
                "gamma0", "gamma0G",
                "gamma1AP", "gamma1APG",
                "delta0", "delta0G",
                "delta1AP", "delta1APG",
                "kappa0", "kappa0G",
                "kappa1PI", "kappa1PIG",
                "kappa2MC",  "kappa2MCG",
                "kappa12MCPI", "kappa12MCPIG")

res.mcmc <- as.mcmc(res)
res.mat.chain1 <- as.matrix(res.mcmc[[1]])
res.mat.chain2 <- as.matrix(res.mcmc[[2]])
res.mat <- as.matrix(res.mcmc)

# DATA FOR EACH CHAIN
str.post.chain1 <- res.mat.chain1[,str.pars]
str.post.chain2 <-  res.mat.chain2[,str.pars]
colnames(str.post.chain2)
dim(str.post.chain2)

# DATA WITH BOTH CHAINS COMBINED
str.post.mat <- res.mat[,str.pars]
colnames(str.post.mat)
dim(str.post.mat)

# CREATE ODDS RATIO MATRIX
post.OR <- apply(str.post.mat, 2, exp)

####### SUMMARIZING POSTERIOR INFORMATION #######
post.means <- apply(str.post.mat, 2, mean)
post.sd <- apply(str.post.mat, 2, sd)
post.lcred <- apply(str.post.mat, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[1])
post.ucred <- apply(str.post.mat, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[2])
# EFFECT SIZE / ODDS RATIO POSTERIOR DISTRBUTION & SUMMARY
post.mean.OR <- apply(post.OR, 2, mean)
post.sd.OR <- apply(post.OR, 2, sd)
post.lcred.OR <- apply(post.OR, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[1])
post.ucred.OR <- apply(post.OR, 2, function(x) 
  quantile(x, probs = c(0.025, 0.975))[2])

post.summary1 <- cbind(post.means,
                       post.sd,
                       post.lcred,
                       post.ucred)
post.summary1 <- apply(post.summary1, 2, function(x) round(x, 2))
Include0 <- ifelse(post.lcred < 0 & post.ucred > 0, "Yes", "No")
post.summary1 <- cbind.data.frame(post.summary1, Include0)
colnames(post.summary1) <- c("Posterior Mean", "Posterior SD",
                             "lb", "ub", "Includes 0")
post.summary1

post.summary.OR <- apply(cbind(post.mean.OR, 
                               post.sd.OR, 
                               post.lcred.OR, 
                               post.ucred.OR), 2, function(x) round(x, 2))
Include1 <- ifelse(post.lcred.OR < 1 & post.ucred.OR > 1, "Yes", "No")
post.summary.OR <- cbind.data.frame(post.summary.OR, Include1)
colnames(post.summary.OR) <- c("Posterior Mean", "Posterior SD",
                               "lb", "ub", "Includes 1")
for(i in 1:nrow(post.summary.OR)){
  if(i %% 2 != 0){
    post.summary.OR[i,] <- ""
  }
}

post.summary <- cbind.data.frame(post.summary1, post.summary.OR)
post.summary

#------------------------------------------------------------------------------------------------------
# ANALYSIS OF  ATTRIBUTE PROFILES ---------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------
att.col.names <- paste0("alpha[", rep(1:nPerson, each = 4), ",", rep(1:nAttributes, each = 1), "]")
names.extract <- colnames(res.mat) %in% att.col.names

att.post.chain1 <- res.mat.chain1[,names.extract]
att.post.chain2 <- res.mat.chain2[,names.extract]

## PUT COLUMNS IN CORRECT ORDER
att.post.chain1 <- att.post.chain1[,att.col.names]
att.post.chain2 <- att.post.chain2[,att.col.names]
colnames(att.post.chain1); all.equal(colnames(att.post.chain1), colnames(att.post.chain2))

att.post.mat <- res.mat[,att.col.names]
colnames(att.post.mat); dim(att.post.mat)

# NUMBER OF OBSERVATIONS THAT HAD ATTRIBUTE MASTERY PROBABILITIES BETWEEN 
# 0.4 and 0.6 (INDIFFERENCE REGION)
sum(colMeans(att.post.mat) >= 0.40 & colMeans(att.post.mat) <= 0.60)
which.indiff <- which(colMeans(att.post.mat) >= 0.40 & colMeans(att.post.mat) <= 0.60)
sum(colMeans(att.post.mat[,which.indiff]) >= 0.40 & colMeans(att.post.mat[,which.indiff]) <= 0.50)
sum(colMeans(att.post.mat[,which.indiff]) >= 0.50 & colMeans(att.post.mat[,which.indiff]) <= 0.60)

# FINAL ATTRIBUTE PROFILE MATRIX (ALPHA)  ###### 
att.vec <- NULL
att.vec <- ifelse(colMeans(att.post.mat) <= 0.5, 0, 1)
ALPHA <- matrix(data = att.vec, nrow = nPerson, ncol = nAttributes, byrow = TRUE)
colnames(ALPHA) <- c("RU", "PI", "AP", "MC")
round(colMeans(ALPHA), 2)

round(cor(ALPHA), 2)

###### MARGINAL ATTRIBUTE MASTERY PROBABILITIES FOR EACH ATTRIBUTE ###### 
### PLOT
marginal.ALPHA <- colMeans(ALPHA)
barplot(marginal.ALPHA, 
                names.arg = colnames(ALPHA),
                ylab = "Proportion", 
                xlab = "Attribute",
                las = 1,
                ylim = c(0, 0.70), 
                cex.names = 1.3,
                cex.axis = 1.2,
                col = "black")

##### UNIQUE ATTRIBUTE PROFILES & PROBABILITIES ###### 
as.data.frame(table(as.data.frame(ALPHA)))
((as.data.frame(table(as.data.frame(ALPHA))))[,5])/nPerson

att.char <- NULL
for(i in 1:16) {
  att.char[i] <- paste0("[",
                        paste(as.matrix(as.data.frame(
                          table(as.data.frame(ALPHA))
                        )[i, -5]), collapse = " "), "]")
}
att.prof.prop <- ((as.data.frame(table(as.data.frame(ALPHA))))[,5])/nPerson

barplot(att.prof.prop, 
                names.arg = att.char,
                ylab = "Proportion",
                las = 1,
                xlab = "Attribute Profile",
                col = "black",
                ylim = c(0, 0.27))

# NUMBER OF TEACHERS WHO FELL INTO FOUR MAJOR PROFILES
sum(as.data.frame(table(as.data.frame(ALPHA)))[which(att.prof.prop > 0.10), 5])
sum(att.prof.prop[which(att.prof.prop > 0.10)])

# NUMBER OF TEACHERS WHO DID NOT FALL INTO FOUR MAJOR PROFILES
sum(as.data.frame(table(as.data.frame(ALPHA)))[-which(att.prof.prop > 0.10), 5])
sum(att.prof.prop[which(att.prof.prop < 0.10)])

###### MASTERY PROFILES FOR AN INDIVIDUAL TEACHERS ###### 
index <- c(1, 3, 135, 258, 456)
for(i in 1:length(index)){
  TID <- index[i] # Teacher ID 
  T.att <- colnames(att.post.mat[,((TID - 1)*4 + 1) : ((TID - 1)*4 + 4) ])
  TID.ALPHA.prob <- colMeans(att.post.mat[,T.att])
  colors <- ifelse(TID.ALPHA.prob <= 0.40, "firebrick", 
                   ifelse(TID.ALPHA.prob >= 0.60, "royalblue", 
                          "gray"))
  xxxx <- barplot(TID.ALPHA.prob, 
                  col = colors, las = 1, names.arg = colnames(ALPHA),
                  ylim = c(0, 1.1), 
                  main = paste0("Attribute Mastery Posterior Probabilites for Teacher ", 
                                TID))
  abline(h = 0.40, lty = 2); abline(h = 0.60, lty = 2)
  yyyy <- TID.ALPHA.prob
  text(x = xxxx, y = yyyy + 0.03,
       labels = as.character(round(yyyy, 2)))
}

######## ANALYSIS OF PSRF
res$BUGSoutput$summary[att.col.names, "Rhat"]
